import sys
import click

from pympl.matrix import mk_random_maxplus, is_reducible
from pympl.utils import serialize



def run(dim, fin, irredu, output):
    A = mk_random_maxplus(dim, fin, irredu)
    r = is_reducible(A)

    if output:
        with open(output, 'wt') as fh:
            serialize(fh, A, r)
    else:
        serialize(sys.stdout, A, r)

@click.command()
@click.option('--dim', type=int, default=3,help='the dimension of the matrix')
@click.option('--fin', type=int, default=3,help='the number of finite elements each row of the matrix')
@click.option('--irredu', type=bool, default=False,help='if True then the matrix is guaranteed to be irreducible')
@click.option('--output', type=str, default=None, help='if set, the produced MPL will be written to the given file name')
def main(dim, fin, irredu, output):
    run(dim, fin, irredu, output)


if __name__ == '__main__':
    main()
