from fractions import Fraction
from random import sample, randint
from pympl.utils import add, iter_max, matrix_max, sub

def add_to_matrix(M, x):
    for r in M:
        for i in range(len(r)):
            r[i] = add(r[i], x)

def diff_eq_to(A, B, x):
    for i in range(len(A)):
        for j in range(len(A[0])):
            if add(A[i][j], x) != B[i][j]:
                return False
    return True

def copy(M):
    return [list(r) for r in M]

def has_none(A):
    check = [any(elem is None for elem in A[i]) for i in range(len(A))]
    return(any(b is True for b in check))

def maxplus_plus(A,B):
    m = len(A)
    n = len(A[0])
    C = [[A[i][j] if B[i][j] is None or (not A[i][j] is None and A[i][j]>=B[i][j]) else B[i][j] for j in range(n)] for i in range(m)]
    return C

def maxplus_times(A, B):
    C = []
    for i in range(len(A)):
        row = []
        for j in range(len(B[0])):
            row.append(iter_max(add(A[i][k], B[k][j]) for k in range(len(A))))
        C.append(row)
    return C

def is_reducible(A):
    B = A
    C = A
    b = has_none(C)
    i = 0
    while i < len(A) and b:
        i += 1
        B = maxplus_times(B,A)
        C = maxplus_plus(C,B)
        b = has_none(C)
    return b

def mk_random_maxplus(dim, fin, must_be_irreducible=False):
    assert dim > 0 and fin > 0, "both arguments must be positive integers"
    assert dim >= fin, "the first argument must be greater than the second one"
    N = range(dim)
    b = True
    A = []
    if not must_be_irreducible:
        for i in N:
            N_i = sample(N, fin)
            A_i = [None if not x in N_i else Fraction(randint(1, 100),randint(1,5)) for x in N]
            A.append(A_i)
        b = is_reducible(A)
    else:
        while(b):
            A = mk_random_maxplus(dim,fin)
            b = is_reducible(A)
    return(A)

def maxplus_period(A,V=None,solver=None):
    it = 0
    if V==None:
        stack = [maxplus_eye(len(A))]
    else:
        stack = [V]
    while True:
        it += 1
        A_new = maxplus_times(A, stack[it-1])
        for i in range(it):
            A_i = stack[i]
            r = sub(matrix_max(A_new), matrix_max(A_i))
            if diff_eq_to(A_i, A_new, r):
                return i, it-i
        stack.append(A_new)

def maxplus_eye(n):
    I = []
    for i in range(n):
        row = []
        for j in range(n):
            row.append(0 if i==j else None)
        I.append(row)
    return I

def maxplus_scaling(A,p):
    B = [[A[i][j]+p if A[i][j] is not None else None for j in range(len(A[0]))] for i in range(len(A))]
    return B

def maxplus_scaling(A,p):
    B = [[A[i][j]+p if A[i][j] is not None else None for j in range(len(A[0]))] for i in range(len(A))]
    return B

def maxplus_pwr(A,n,cache=None):
    if (n==0):
        return maxplus_eye(len(A))
    elif (n==1):
        return A
    else:
        if cache and n in cache:
            return cache[n]
        n1 = n//2
        A1 = maxplus_pwr(A, n1)
        n2 = n - n1
        A2 = maxplus_pwr(A, n2)
        res = maxplus_times(A1, A2)
        if cache:
            cache[n] = res
        return res

def normalization(A, v):
    res = []
    max_cols = [None for x in range(len(A))]
    for i, row in enumerate(A):
        n_row = []
        for j,x in enumerate(row):
            y = sub(x, v[i])
            n_row.append(y)
            if max_cols[j] is None or (y is not None and y > max_cols[j]):
                max_cols[j] = y
        res.append(n_row)
    return res, max_cols

def is_solvable(A, v):
    A, mc = normalization(A, v)
    x = [[-i] for i in mc]
    y = maxplus_times(A, x)
    if all(i == 0 for i in y):
        return True, x
    else:
        return False, None

def back_simulate(A,s0):
    it = 0
    while True:
        b, v = is_solvable(A,s0)
        if b == True:
            it += 1
            s0 = v
        else:
            return it

def finite_loc(A,dim=0):
    if dim==0:
        idx = [[j for j in range(len(A[i])) if A[i][j] is not None] for i in range(len(A))]
    elif dim==1:
        idx = [[j for j in range(len(A[i])) if A[j][i] is not None] for i in range(len(A))]
    else:
        print("error")
    return idx

def intersection(lst1,lst2):
    return [value for value in lst1 if value in lst2]

def argmax(lst):
    val = None
    L = [[lst[i] for i in range(len(lst)) if lst[i] is not None],
         [i for i in range(len(lst)) if lst[i] is not None]]
    if len(L[0])==0:
        return None,[]
    else:
        val = max(L[0])
        idx = [L[1][i] for i in range(len(L[0])) if L[0][i]==val]
    return val, idx

def critical_nodes(A):
    m = len(A)
    res = argmax([A[j][j] for j in range(m)])
    w = None
    l = []
    N = []
    if res[0] is not None:
        N, w, l = [res[1]], Fraction(res[0],1), [1]
    B = [[A[i][j] if i!=j else None for j in range(m)] for i in range(m)]
    for i in range(2,m+1):
        B = maxplus_times(B,A)
        res = argmax([B[j][j] for j in range(m)])
        if res[0] is not None and (w is None or w < Fraction(res[0],i)):
            N, w, l = [res[1]], Fraction(res[0],i), [i]
        elif res[0] is not None and w == Fraction(res[0],i) and i<=len(res[1]):
            N.append(res[1])
            l.append(i)
        for j in range(m):
            B[j][j] = None
    return N, w, l

def cyclicity(A):
    N, _, l = critical_nodes(A)
    if len(l)==1:
        return l[0]
    else:
        m = len(l)
        stop = False
        for i in range(m-1):
            for j in range(i+1,m):
                if len(intersection(N[i],N[j]))>0:
                    l[i] = gcd(l[i],l[j])
                    l[j] = 0
        l = [x for x in l if x>0]
        for i in range(1,len(l)):
            l[0] = lcm(l[0],l[i])
        return l[0]

def cycle_mean(A):
    m = len(A)
    w, _ = argmax([A[j][j] for j in range(m)])
    B = [[A[i][j] if i!=j else None for j in range(m)] for i in range(m)]
    for i in range(2,m+1):
        B = maxplus_times(B,A)
        w_i, _ = argmax([B[j][j] for j in range(m)])
        if w_i is not None and (w is None or w < Fraction(w_i,i)):
            w = Fraction(w_i,i)
        for j in range(m):
            B[j][j] = None
    return w

def eigenspace(A):
    m = len(A)
    l = cycle_mean(A)
    A_l = maxplus_scaling(A,-l)
    M = A_l
    B = M
    for i in range(1,m):
        M = maxplus_times(M,A_l)
        B = maxplus_plus(B,M)
    E = []
    for i in range(m):
        r = [B[j][i] for j in range(m)]
        if all(not elem is None for elem in r) and r[i]==0:
            E.append(r)
    if len(E)>0:
        E_t = [[E[j][i] for j in range(len(E))] for i in range(len(E[0]))]
        E = E_t
    return(E)
