import json
from fractions import Fraction


def gcd(a,b):
    """Compute the greatest common divisor of a and b"""
    while b > 0:
        a, b = b, a % b
    return a


def lcm(a, b):
    """Compute the lowest common multiple of a and b"""
    return a * b // gcd(a, b)


def add(x, y):
    if x is None or y is None:
        return None
    return x+y


def sub(x, y):
    if x is None or y is None:
        return None
    return x-y


def matrix_max(A):
    res = A[0][0]
    for row in A:
        for x in row:
            if res is None or (x is not None and x > res):
                res = x
    return res


def iter_max(args):
    res = next(args)
    for a in args:
        if res is None or (a is not None and a > res):
            res = a
    return res


def serialize(fh, A, is_reducible, k0=None, c=None):
    res = {}
    res['reducible'] = is_reducible
    res['matrix'] = [[float(x) if x is not None else '-inf' for x in row] for row in A]
    if k0: res['k0'] = k0
    if c: res['c'] = c
    json.dump(res, fh, indent=2)

def deserialize(fh):
    j = json.load(fh)
    reducible = j['reducible']
    A = [[Fraction(x) if x != '-inf' else None for x in row] for row in j['matrix']]
    k0, c = j.get('k0', None), j.get('c', None)
    return A, reducible, k0, c
