import sys
import time
import io
import click

from pysmt.shortcuts import get_env
from pysmt.logics import QF_LRA

from pympl.matrix import maxplus_period
from pympl.smt import smt_period_single_vars, smt_period_basic, smt_period_rdl
from pympl.smt import smt_period_sym_matrix, smt_period_single_vars_distrib
from pympl.utils import deserialize


def run_test(fh, name, function, A, solver=None):
    fh.write("\nComputation using %s\n" % name)
    t1 = time.time()
    mat_res = function(A, solver=solver)
    t2 = round(time.time() - t1,3)
    fh.write(" k0 = %s\n" % mat_res[0])
    fh.write(" c = %s\n" % mat_res[1])
    fh.write(" time: %s seconds \n" % t2)


def run(buf, ifile, solver):
    if ifile:
        with open(ifile) as fh:
            A,b1,_,_ = deserialize(fh)
    else:
        A,b1,_,_ = deserialize(sys.stdin)

    output = ''

    if not b1:
        buf.write('The matrix is irreducible\n')
    else:
        buf.write('The matrix is reducible\n')


    run_test(buf, "matrix-multiplication", maxplus_period, A, solver)
    #run_test(fh, "SMT no-vars distributed", smt_period_single_vars_distrib, A, solver)
    #run_test(fh, "SMT no-vars compact", smt_period_single_vars, A, solver)
    run_test(buf, "SMT + DL", smt_period_rdl, A, solver)
    #run_test(fh, "SMT 3-BMC", smt_period_basic, A, solver)
    #run_test(fh, "SMT Symbolic Matrix", smt_period_sym_matrix, A, solver)


def run_get_output(ifile, solver):
    with io.StringIO() as buf:
        run(buf, ifile, solver)
        return buf.getvalue()

@click.command()
@click.argument('ifile', type=str, default=None, required=False)
@click.option('--solver',
              type=click.Choice(get_env().factory.all_solvers(logic=QF_LRA)),
              default=None)
def main(ifile, solver):
    run(sys.stdout, ifile, solver)

if __name__ == '__main__':
    main()
