import solver
import generate
import random
import uuid
import re
import shutil
import os
import signal
import click

STOP = False

def signal_handler(sig, frame):
    print('You pressed Ctrl+C!')
    global STOP
    STOP = True

def timeout_handler(sig, frame):
    print('Timeout!')
    raise Exception("timeout")

@click.command()
@click.option('--dim', type=int, default=None, help='the dimension of the matrix. If omitted a random value between 1 and 50 will be picked for each generated problem.')
@click.option('--fin', type=int, default=None, help='the number of finite elements each row of the matrix. If omitted, a random value between 1 and dim will be picked for each generated problem.')
@click.option('--num', type=int, default=100, help='the number of experiments')
@click.argument('output_dir', type=click.Path(exists=False))
def main(output_dir, dim, fin, num):
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGALRM, timeout_handler)

    if not os.path.exists(output_dir):
        os.mkdir(output_dir)

    odim = dim
    ofin = fin

    n = 0
    while n < num and not STOP:
        if odim is None:
            dim = random.randint(1, 50)
        if ofin is None:
            fin = random.randint(1, dim)
        n += 1


        problem_id = str(uuid.uuid1())

        problem_name = 'problem_'+problem_id+'.txt'
        result_name = 'result_'+problem_id+'.txt'

        generate.run(dim, fin, True, problem_name)

        signal.alarm(3600)
        try:
            output = solver.run_get_output(problem_name, None)
            signal.alarm(0)
        except Exception as e:
            os.remove(problem_name)
            continue

        m = re.search(r'k0 = (\d+)', output)

        k0 = str(m.group(1))

        k0_dir = os.path.join(output_dir, str(dim))
        if not os.path.exists(k0_dir):
            os.mkdir(k0_dir)

        shutil.move(problem_name, os.path.join(k0_dir, problem_name))

        result = os.path.join(k0_dir, result_name)
        with open(result, 'wt') as fh:
            fh.write(output)
        #print('Problem and result with UUID %s moved in %s' % (problem_id, k0_dir))

    print('Done')

if __name__ == '__main__':
    main()
